/*
     .
      :
RELATIONS_IN_DATABASE==3
FRAGMENTATION_ATTRIBUTES=={1, 1, 2}
@memo    
@name AgentManager_Check
@author .. , .. 
@version 1.0
@see AgentManager
*/




#include "errcode.h"	/*   */
#include "compiler.h"	/*   */
#include "lineplan.h"	/*    */
#include "parallel.h"	/*   */
#include "agentmgr.h"	/*  */
#include "dbconfig.h"	/*     */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>


/*    */
static void CheckParallelAgent(treenode_t *);
/*    */
static void PrintParallelAgent(treenode_t *);


/*
@memo   
*/
#define MYNODE	0

/*
@memo   
*/
#define TEST_QUERY_QTY	4

/*
@memo       
*/
#define QUERY_LINE_MAX	(1<<TREE_LEVEL_MAX)

/*
@memo  
*/
static const char * TEST_QUERY[TEST_QUERY_QTY][QUERY_LINE_MAX]={
	/* JOIN(1, SCAN(0), SCAN(1)) */
	{
	"0 J 1 #0 #1",
	"",	"", "", "",	"", "", "",	"", "", "",	"", "", "",	"", ""
	},

	/* JOIN(1, SCAN(2), SCAN(0)) */
	{
	"0 J 1 #2 #0",
	"",	"", "", "",	"", "", "",	"", "", "",	"", "", "",	"", ""
	},

	/* JOIN(2, JOIN(2, SCAN(1), RESTRICT(1, <, 31, SCAN(3))), RESTRICT(2, >, 19, SCAN(0))) */
	{
	"0 J 2 1 2",
	"1 J 2 #1 3",
	"2 R 2 > 19 #0",
	"3 R 1 < 31 #2",
	"",	"", "", "",	"", "", "",	"", "", "",	"", ""
	},

	/* JOIN(3, JOIN(2, RESTRICT(1, >, 10, SCAN(0)), RESTRICT(2, <, 8, SCAN(2))), JOIN(3, SCAN(2), RESTRICT(1, =, 7, JOIN(3, SCAN(3), RESTRICT(2, < ,4, SCAN(1)))))) */
	{
	"0 J 3 1 2",
	"1 J 2 3 4",
	"2 J 3 #2 5",
	"3 R 1 > 10 #0",
	"4 R 2 < 8 #2",
	"5 R 1 = 7 6",
	"6 J 3 #2 7",
	"7 R 2 < 4 #1",	
	"", "", "",	"", "", "",	"", ""
	}
};

/*
@memo     ( ",  ,  ")
*/
static const char * TEST_AGENT[TEST_QUERY_QTY]={
	/* JOIN(1, SCAN(0), SCAN(1)) */
	"STORE MERGE GATHER 0 SPLIT JOIN 1 SCAN 0 SCAN 1 SCATTER ",

	/* JOIN(1, SCAN(2), SCAN(0)) */
	"STORE MERGE GATHER 0 SPLIT JOIN 1 MERGE GATHER 1 SPLIT SCAN 2 SCATTER SCAN 0 SCATTER ",

	/* JOIN(2, JOIN(2, SCAN(1), RESTRICT(1, <, 31, SCAN(3))), RESTRICT(2, >, 19, SCAN(0))) */
	"STORE MERGE GATHER 0 SPLIT JOIN 2 JOIN 2 MERGE GATHER 1 SPLIT SCAN 1 SCATTER RESTRICT 1 < 31 SCAN 2 MERGE GATHER 2 SPLIT RESTRICT 2 > 19 SCAN 0 SCATTER SCATTER ",

	/* JOIN(3, JOIN(2, RESTRICT(1, >, 10, SCAN(0)), RESTRICT(2, <, 8, SCAN(2))), JOIN(3, SCAN(2), RESTRICT(1, =, 7, JOIN(3, SCAN(3), RESTRICT(2, < ,4, SCAN(1)))))) */
	"STORE MERGE GATHER 0 SPLIT JOIN 3 MERGE GATHER 2 SPLIT JOIN 2 MERGE GATHER 1 SPLIT RESTRICT 1 > 10 SCAN 0 SCATTER RESTRICT 2 < 8 SCAN 2 SCATTER JOIN 3 MERGE GATHER 3 SPLIT SCAN 2 SCATTER RESTRICT 1 = 7 JOIN 3 MERGE GATHER 4 SPLIT SCAN 2 SCATTER MERGE GATHER 5 SPLIT RESTRICT 2 < 4 SCAN 1 SCATTER SCATTER "
};

/*
@memo 
*/
static treenode_t * AGENT;

/*
@memo     
*/
static char agentstr[1024];



int main(void)
{
	int i, j, res;
	FILE * f;
	treenode_t * tree;

	printf("Test Manager of parallel agents: START\n");

	printf("\tInit database and dictionary: START\n");
	res=db_InitDatabase(MYNODE, NULL);
	assert(res==SUCCESS);
	printf("\tInit database and dictionary: STOP\n");

	printf("\tInit Compiler: START\n");
	res=cp_InitCompiler(MYNODE);
	assert(res==SUCCESS);
	printf("\tInit Compiler: STOP\n");

	printf("\tInit Plan Generator: START\n");
	res=lp_InitPlanGenerator(MYNODE);
	assert(res==SUCCESS);
	printf("\tInit Plan Generator: STOP\n");

	printf("\tInit Parallelizer: START\n");
	res=pl_InitParallelizer(MYNODE);
	assert(res==SUCCESS);
	printf("\tInit Parallelizer: STOP\n");

	printf("\tInit Agent Manager: START\n");
	res=am_InitAgentManager(MYNODE);
	assert(res==SUCCESS);
	res=am_InitAgentManager(MYNODE); /*   */
	assert(res==SUCCESS);
	printf("\tInit Agent Manager: STOP\n");

	printf("\tCheck functions to make parallel agents: START\n");
	for (i=0; i<TEST_QUERY_QTY; i++) {
		printf("\t\tcase #%d of %d: START\n", i+1, TEST_QUERY_QTY);
		f=fopen(QUERY_FNAME, "wt");
		assert(f!=NULL);
		for (j=0; j<QUERY_LINE_MAX; j++)
			if (strlen(TEST_QUERY[i][j])>0)
				fprintf(f, "%s\n", TEST_QUERY[i][j]);
			else
				break;
		fclose(f);
		strcpy(agentstr, "");
		tree=cp_CompileQuery(QUERY_FNAME);
		assert(tree!=NULL);
		res=lp_GeneratePlan(tree);
		assert(res==SUCCESS);
		res=pl_ParallelizeQuery(tree);
		assert(res==SUCCESS);

		AGENT=am_MakeAgent(tree);
		assert(AGENT!=NULL);

		CheckParallelAgent(AGENT);
		strcpy(agentstr, "");
		PrintParallelAgent(AGENT);
/*
		printf("\nShould be\n'%s'\nIn fact\n'%s'\n", TEST_AGENT[i], agentstr);
*/		
		assert(strcmp(agentstr, TEST_AGENT[i])==0);
		printf("\t\tcase #%d of %d: STOP\n", i+1, TEST_QUERY_QTY);
	}
	printf("\tCheck functions to make parallel agents: STOP\n");

	printf("Test Manager of parallel agents: STOP\n");

	return 0;
}

/*
@memo   
@param node  
@param root    
*/
static void CheckParallelAgent(treenode_t * root)
{
	assert(root!=NULL);
	assert(root->operation==STORE);
	assert(root->executefn==ExecuteSTORE);

	assert(root->left->operation==MERGE);
	assert(root->left->executefn==ExecuteMERGE);
	assert(root->left->resetfn==ResetMERGE);

	assert(root->left->left->operation==GATHER);
	assert(root->left->left->executefn==ExecuteGATHER);
	assert(root->left->left->resetfn==ResetGATHER);
	assert(opGATHER(root->left->left, port)==0);

	assert(root->left->right->operation==SPLIT);
	assert(root->left->right->executefn==ExecuteSPLIT);
	assert(root->left->right->resetfn==ResetSPLIT);
	assert(opSPLIT(root->left->right, distributefn)==am_PMergeFn);

	assert(root->left->right->right->operation==SCATTER);
	assert(root->left->right->right->executefn==ExecuteSCATTER);
	assert(root->left->right->right->resetfn==ResetSCATTER);
	assert(opSCATTER(root->left->right->right, port)==0);
}

/*
@memo   
@param root    
*/
static void PrintParallelAgent(treenode_t * root)
{
	char buf[10];

	if (root==NULL)
		return;
	switch (root->operation) {
	case JOIN:
		strcat(agentstr, "JOIN");
		strcat(agentstr, " ");
		sprintf(buf, "%d", opJOIN(root, attribute));
		strcat(agentstr, buf);
		strcat(agentstr, " ");
		break;
	case RESTRICT:
		strcat(agentstr, "RESTRICT");
		strcat(agentstr, " ");
		sprintf(buf, "%d", opJOIN(root, attribute));
		strcat(agentstr, buf);
		strcat(agentstr, " ");
		switch (opRESTRICT(root, theta)) {
		case EQUAL: 
			strcat(agentstr, "=");
			break;
		case NOT_EQUAL: 
			strcat(agentstr, "<>");
			break;
		case GREATER: 
			strcat(agentstr, ">");
			break;
		case LESS: 
			strcat(agentstr, "<");
			break;
		case GREATER_OR_EQUAL: 
			strcat(agentstr, ">=");
			break;
		case LESS_OR_EQUAL: 
			strcat(agentstr, "<=");
			break;
		default: 
			assert(0);
		}
		strcat(agentstr, " ");
		sprintf(buf, "%d", opRESTRICT(root, literal));
		strcat(agentstr, buf);
		strcat(agentstr, " ");
		break;
	case SCAN: 
		strcat(agentstr, "SCAN");
		strcat(agentstr, " ");
		sprintf(buf, "%d", opSCAN(root, relation));
		strcat(agentstr, buf);
		strcat(agentstr, " ");
		break;
	case MERGE:
		strcat(agentstr, "MERGE");
		strcat(agentstr, " ");
		assert(root->executefn==ExecuteMERGE);
		assert(root->left->operation==GATHER);
		assert(root->right->operation==SPLIT);
		break;
	case GATHER:
		strcat(agentstr, "GATHER");
		strcat(agentstr, " ");
		sprintf(buf, "%d", opGATHER(root, port));
		strcat(agentstr, buf);
		strcat(agentstr, " ");
		assert(root->executefn==ExecuteGATHER);
		assert(root->left==NULL);
		assert(root->right==NULL);
		break;
	case SPLIT:
		strcat(agentstr, "SPLIT");
		strcat(agentstr, " ");
		assert(root->executefn==ExecuteSPLIT);
		assert(root->right->operation==SCATTER);
		assert(root->left->operation==JOIN || root->left->operation==RESTRICT || root->left->operation==SCAN);
		break;
	case SCATTER:
		strcat(agentstr, "SCATTER");
		strcat(agentstr, " ");
		assert(root->executefn==ExecuteSCATTER);
		assert(root->left==NULL);
		assert(root->right==NULL);
		break;
	case STORE:
		strcat(agentstr, "STORE");
		strcat(agentstr, " ");
		break;
	default:
		assert(0);
	}
	PrintParallelAgent(root->left);
	PrintParallelAgent(root->right);
}