/*
     
@memo      
@name PlanGenerator_Check
@author .. , .. 
@version 1.0
@see PlanGenerator
*/


#include "errcode.h"		/*   */
#include "lineplan.h"		/*  */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>


static void CheckPlan(treenode_t *); /*     */

/*
@memo  ,    
*/
#define MYNODE	0

/*
@memo   
*/
#define TEST_QUERY_QTY	3

/*
@memo       
*/
#define QUERY_LINE_MAX	(1<<TREE_LEVEL_MAX)

/*
@memo  
*/
static const char * TEST_QUERY[TEST_QUERY_QTY][QUERY_LINE_MAX]={
	/* JOIN(1, SCAN(0), SCAN(1)) */	
	{ 
	"0 J 1 #0 #1", 
	"",	"", "", "",	"", "", "",	"", "", "",	"", "", "",	"", ""
	},

	/* JOIN(3, JOIN(1, SCAN(2), RESTRICT(1, <, 31, SCAN(2))), RESTRICT(2, >, 19, SCAN(1))) */
	{
	"0 J 3 1 2",
	"1 J 1 #2 3",
	"2 R 2 > 19 #1",
	"3 R 1 < 31 #2",
	"",	"", "", "",	"", "", "",	"", "", "",	"", ""
	},

	/* JOIN(3, JOIN(1, RESTRICT(1, <, 31, SCAN(0)), RESTRICT(2, =, 10, SCAN(1))), JOIN(2, RESTRICT(3, <, 90, SCAN(2)), RESTRICT(0, >, 4, SCAN(2)))) */
	{
	"0 J 3 1 2",
	"1 J 1 3 4",
	"2 J 2 5 6",
	"3 R 1 < 31 #0",
	"4 R 2 = 10 #1",	
	"5 R 3 < 90 #2", 
	"6 R 0 > 4 #2", 
	"",	"", "", "",	"", "", "",	"", ""
	}
};

/*
@memo     ( ",  ,  ")
*/
static const char * TEST_TREE[TEST_QUERY_QTY]={
	/* JOIN(1, SCAN(0), SCAN(1)) */	
	"JOIN 1 [7] SCAN 0 [4] SCAN 1 [4] ",

	/* JOIN(3, JOIN(1, SCAN(2), RESTRICT(1, <, 31, SCAN(3))), RESTRICT(2, >, 19, SCAN(1))) */
	"JOIN 3 [10] JOIN 1 [7] SCAN 2 [4] RESTRICT 1 < 31 [4] SCAN 2 [4] RESTRICT 2 > 19 [4] SCAN 1 [4] ",

	/* JOIN(3, JOIN(1, RESTRICT(1, <, 31, SCAN(0)), RESTRICT(2, =, 10, SCAN(1))), JOIN(2, RESTRICT(3, <, 90, SCAN(2)), RESTRICT(0, >, 4, SCAN(3)))) */
	"JOIN 3 [13] JOIN 1 [7] RESTRICT 1 < 31 [4] SCAN 0 [4] RESTRICT 2 = 10 [4] SCAN 1 [4] JOIN 2 [7] RESTRICT 3 < 90 [4] SCAN 2 [4] RESTRICT 0 > 4 [4] SCAN 2 [4] "
};


int main(void)
{
	int i, j, res;
	FILE * f;
	treenode_t * tree;

	printf("Test Linear Plan Generator: START\n");
	printf("\tInit Database and dictionary: START\n");
	res=db_InitDatabase(MYNODE, NULL);
	assert(res==SUCCESS);
	printf("\tInit Database and dictionary: STOP\n");
	printf("\tInit Compiler: START\n");
	res=cp_InitCompiler(MYNODE);
	assert(res==SUCCESS);
	printf("\tInit Compiler: STOP\n");

	printf("\tInit Plan Generator: START\n");
	res=lp_InitPlanGenerator(MYNODE);
	assert(res==SUCCESS);
	res=lp_InitPlanGenerator(MYNODE); /*   */
	assert(res==SUCCESS);
	printf("\tInit Plan Generator: STOP\n");

	printf("\tCheck functions to make linear plan: START\n");
	for (i=0; i<TEST_QUERY_QTY; i++) {
		printf("\t\tcase #%d of %d: START\n", i+1, TEST_QUERY_QTY);
		f=fopen(QUERY_FNAME, "wt");
		assert(f!=NULL);
		for (j=0; j<QUERY_LINE_MAX; j++)
			if (strlen(TEST_QUERY[i][j])>0)
				fprintf(f, "%s\n", TEST_QUERY[i][j]);
			else
				break;
		fclose(f);
		tree=cp_CompileQuery(QUERY_FNAME);
		assert(tree!=NULL);
		res=lp_GeneratePlan(tree);
		assert(res==SUCCESS);
		CheckPlan(tree);
		printf("\t\tcase #%d of %d: STOP\n", i+1, TEST_QUERY_QTY);
	}
	printf("\tCheck functions to make linear plan: STOP\n");


	printf("Test Linear Plan Generator: STOP\n");

	return 0;
}

/*
    ",  ,  "
@memo   
@param root    
*/
static void CheckPlan(treenode_t * root)
{
	if (root==NULL)
		return; 
	switch (root->operation) {
	case JOIN: 
		assert(root->executefn==ExecuteJOIN_NestedLoops);
		assert(root->resetfn==ResetJOIN);
		break;
	case RESTRICT: 
		assert(root->executefn==ExecuteRESTRICT);
		assert(root->resetfn==ResetRESTRICT);		
		break;
	case SCAN: 
		assert(root->executefn==ExecuteSCAN);
		assert(root->resetfn==ResetSCAN);
		break;
	default: 
		assert(0);
	}
	CheckPlan(root->left);
	CheckPlan(root->right);
}