#ifndef _DEBUG_H
#define _DEBUG_H

/**
  
@memo 
@name Debugger
@author (c) 2004 .. , .. 
@version 1.0  \today
*/

#include "dbgen.h"

#define NDEBUG
#undef NDEBUG


/*@{*/

/**
   
@memo  
@name Debugger_Constants
*/

/*@{*/


/*@}*/

/**
   
@memo  
@name Debugger_Functions
*/

/*@{*/

/**
@memo  
@param node   
*/
#ifndef NDEBUG

#define DBG_INIT(node)	dbg_Init(node)

#else

#define DBG_INIT(node)	((void) 0)

#endif

/**
@memo    
@param node  
*/
#ifndef NDEBUG

#define DBG_VISIBLE(node)	dbg_Visible(node)

#else

#define DBG_VISIBLE(node)	((void) 0)

#endif

/**
@memo       
@param node  
*/
#ifndef NDEBUG

#define DBG_VISIBLE_ALL	dbg_VisibleAll()

#else

#define DBG_VISIBLE_ALL	((void) 0)

#endif

/**
@memo    
@param node  
*/
#ifndef NDEBUG

#define DBG_INVISIBLE(node)	dbg_Invisible(node)

#else

#define DBG_INVISIBLE(node)	((void) 0)

#endif

/**
@memo       
@param node  
*/
#ifndef NDEBUG

#define DBG_INVISIBLE_ALL	dbg_InvisibleAll()

#else

#define DBG_INVISIBLE_ALL	((void) 0)

#endif

/**
@memo  
@param tuple   
@param ary  
*/
#ifndef NDEBUG

#define TUPLE(tuple, ary)	dbg_PrintTuple(tuple, ary)

#else

#define TUPLE(tuple, ary)	((void) 0)

#endif

/*
@memo  :  
*/
#define MAKE_STMT(stmt)	do { stmt } while (0)

/**
       :
\begin{verbatim}
[_]	_	__	_
\end{verbatim}
       .  :
\begin{verbatim}
WATCH(("myfunc\tint_arg==%d\tstr_arg==%s\tptr_arg==%p", int_arg, str_arg, ptr_arg));
\end{verbatim}
       "\n" ( ).
     ,      
 .
@memo   
@see DBG_SCALE START STOP RETURN
*/
#ifndef NDEBUG

#define WATCH(point_fmtstr_arglist)		\
MAKE_STMT( \
	if (dbg_IsVisible(dbg_WhoAmI())) { \
		printf("<%d>\t%ld\t", dbg_WhoAmI(), dbg_GetDbgCnt()); \
		printf point_fmtstr_arglist ; \
		printf(" (file %s, line %d)\n", /*__FILE__*/"?", __LINE__); \
		fflush(stdout); \
	} ; \
)

#else

#define WATCH(point_fmtstr_arglist)	((void) 0)

#endif

/**
       :
\begin{verbatim}
[_]	_	_: START
\end{verbatim}
     ,      
 .
@memo     
@see DBG_SCALE WATCH STOP RETURN
*/
#ifndef NDEBUG

#define START(funcname)		\
MAKE_STMT( \
	if (dbg_IsVisible(dbg_WhoAmI())) { \
		printf("<%d>\t%ld\t%s: START (file %s, line %d)\n", dbg_WhoAmI(), dbg_GetDbgCnt(), funcname, /*__FILE__*/ "?", __LINE__); \
		fflush(stdout); \
	} ; \
)

#else

#define START(funcname)		((void) 0)

#endif

/**
       :
\begin{verbatim}
[_]	_	_: STOP
\end{verbatim}
     ,      
 .
@memo     
@see DBG_SCALE WATCH START RETURN
*/
#ifndef NDEBUG

#define STOP(funcname)		\
MAKE_STMT( \
	if (dbg_IsVisible(dbg_WhoAmI())) { \
		printf("<%d>\t%ld\t%s: STOP (file %s, line %d)\n", dbg_WhoAmI(), dbg_GetDbgCnt(), funcname, /*__FILE__*/ "?", __LINE__); \
		fflush(stdout); \
	} ; \
)

#else

#define STOP(funcname)		((void) 0)

#endif

/**
       :
\begin{verbatim}
[_]	_	_: RETURN 
\end{verbatim}
     ,      
 .
@memo   
@see DBG_SCALE WATCH START RETURN_INT RETURN_LONG RETURN_STR RETURN_CHAR RETURN_PTR RETURN_NULL
*/
#ifndef NDEBUG

#define RETURN(funcname, fmtstr, result)		\
MAKE_STMT( \
	if (dbg_IsVisible(dbg_WhoAmI())) { \
		printf("<%d>\t%ld\t%s: RETURN ", dbg_WhoAmI(), dbg_GetDbgCnt(), funcname); \
		printf(fmtstr, result); \
		printf("(file %s, line %d)\n", /*__FILE__*/ "?", __LINE__); \
		fflush(stdout); \
	} ; \
)

#else

#define RETURN(funcname, fmtstr, result)		((void) 0)

#endif

/**
@memo   int-
*/
#define RETURN_INT(funcname, result)	RETURN(funcname, "%d", result)

/**
@memo   long-
*/
#define RETURN_LONG(funcname, result)	RETURN(funcname, "%ld", result)

/**
@memo    
*/
#define RETURN_STR(funcname, result)	RETURN(funcname, "%s", result)

/**
@memo   char-
*/
#define RETURN_CHAR(funcname, result)	RETURN(funcname, "%c", result)

/**
@memo   pointer-
*/
#define RETURN_PTR(funcname, result)	RETURN(funcname, "%p", result)

/**
@memo   NULL
*/
#define RETURN_NULL(funcname)	RETURN(funcname, "%s", "NULL")

/**
@memo   -EOF
*/
#define RETURN_EOF(funcname)	RETURN(funcname, "%s", "EOFtuple")

/**
@memo    -EOF
*/
#define RETURN_notEOF(funcname)	RETURN(funcname, "%s", "not EOFtuple")


/**
@memo    
*/
#define RETURN_SUCCESS(funcname)	RETURN(funcname, "%s", "SUCCESS")

/**
@memo    
*/
#define RETURN_ESRCH(funcname)	RETURN(funcname, "%s", "ESRCH")

/**
@memo    
*/
#define RETURN_EINVAL(funcname)	RETURN(funcname, "%s", "EINVAL")

/**
@memo    
*/
#define RETURN_ENOINIT(funcname)	RETURN(funcname, "%s", "ENOINIT")

/**
@memo   
*/
#define RETURN_EOVRFLOW(funcname)	RETURN(funcname, "%s", "EOVRFLOW")

/**
@memo     
*/
#define RETURN_ENOMEM(funcname)	RETURN(funcname, "%s", "ENOMEM")

/*@}*/

/*@}*/

extern void dbg_Init(int);
extern int dbg_IsVisible(int);
extern void dbg_Visible(int);
extern void dbg_VisibleAll(void);
extern void dbg_Invisible(int);
extern void dbg_InvisibleAll(void);
extern int dbg_WhoAmI(void);
extern void dbg_PrintTuple(tuple_t, int);
extern unsigned long dbg_GetDbgCnt(void);

#endif /* _DEBUG_H */