/*
    
@memo  
@name  Bracket_Implementation
@author .. , .. 
@version 1.0
@see Engine
*/

#include "bracket.h"	/*  */
#include "filemgr.h"	/*   */
#include "errcode.h"	/*   */
#include "common.h"		/*     */
#include "lineplan.h"	/*    */
#include "agentmgr.h"	/*    */
#include "dbgen.h"		/*    */
#include "parallel.h"	/*   */
#include "debug.h"		/*    */
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>


/*@{*/

/*
     
@name Bracket_Implementation_Types
@memo    
@see Bracket_Implementation_Functions 
*/

/*@{*/

/*
@memo  
*/
static dictionary_t * DICTIONARY;

/*
@memo   
*/
static int initflag=FALSE;


/*@}*/

/*
     
@name Bracket_Implementation_Functions
@memo    
@see Bracket_Implementation_Types
*/

/*@{*/


/*
       
         .
       .
@memo   
@param node	   	
@return SUCCESS       
@see br_Reset br_Next
*/
int br_Init(int node)
{
	DBG_INIT(node);
	START("br_Init");
	if (initflag)
		return SUCCESS;
	if (node<0 || node>=FRAGMENTS_IN_RELATION)
		return EINVAL;
	if ((DICTIONARY=db_DatabaseDictionary())==NULL)
		return EINVAL;
	if (node!=DICTIONARY->whoami)
		return EINVAL;
	initflag=TRUE;   
	STOP("br_Init");
	return SUCCESS;
}

/*
       .
@memo   
@param node      
@see treenode_t br_Next
*/
void br_Reset(treenode_t * node)
{
	int res;
	
	START("br_Reset");
	switch (node->operation) {
	case STORE:
		WATCH(("br_Reset for node STORE"));
		break;
	case JOIN:
		WATCH(("br_Reset for node JOIN"));
		break;
	case RESTRICT:
		WATCH(("br_Reset for node RESTRICT"));
		break;
	case SCAN:
		WATCH(("br_Reset for node SCAN"));
		break;
	case MERGE:
		WATCH(("br_Reset for node MERGE"));
		break;
	case GATHER:
		WATCH(("br_Reset for node GATHER"));
		break;
	case SPLIT:
		WATCH(("br_Reset for node SPLIT"));
		break;
	case SCATTER:
		WATCH(("br_Reset for node SCATTER"));
		break;
	default:
		assert(0);
	}
	node->isBOF=TRUE;
	node->isEOF=FALSE;
	res=(node->resetfn)(node);
	assert(res==SUCCESS);
	if (node->left!=NULL)	
		br_Reset(node->left);
	if (node->right!=NULL)
		br_Reset(node->right);
	STOP("br_Reset");
}

/*
          , 
    .
@memo    
@param node      
@return      NULL
@see tuple_t treenode_t br_Reset
*/
tuple_t br_Next(treenode_t * node)
{
	tuple_t tuple;

	START("br_Next");
	node->isBOF=FALSE;
	tuple=(node->executefn)(node);
	if (IsEOF(tuple))
		node->isEOF=TRUE;
	STOP("br_Next");
	return tuple;
}


/*@}*/

/*@}*/

