/*
      
@memo   
@name DatabaseGenerator_Implementation
@author .. , .. 
@version 1.0
*/


#include "dbgen.h"		/*  */
#include "errcode.h"	/*   */
#include "filemgr.h"	/*   */
#include "common.h"		/*     */
#include "debug.h"		/*    */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>


/*@{*/

/*
       
@name DatabaseGenerator_Implementation_Variables
@memo    
@see DatabaseGenerator_Implementation_Functions
*/

/*@{*/

/*
@memo   
*/
static dictionary_t DICTIONARY;

/*
@memo  
*/
static int initflag=FALSE;

/*
-EOF
@memo -EOF
@see IsEOF
*/
tuple_t EOFtuple;

/*@}*/

/*
       
@name DatabaseGenerator_Implementation_Functions
@memo    
@see DatabaseGenerator_Implementation_Constants
*/

/*@{*/

/**
  (    )  ,    
 -EOF,     . NB!     , 
 ==NULL.
 " "   ,     .
    -EOF,        
 domain_t  EOFmarker.
@memo    EOF
@param tuple   
@return 1    -EOF,  0
@see fm_NextTuple tuple_t domain_t EOFmarker
*/
int IsEOF(tuple_t tuple)
{
	return (tuple==NULL ? FALSE : tuple[0]==EOFmarker);
}

/*
            .
.
1.        ( RELATIONS_FILENAMES).
        ,     .
2.      ATTRIBUTES_IN_RELATION    domain_t,
    .
3.              
      .
4.  ,     0,      
   0..DOMAIN_VALUE_MAX-1. 
5.    0        (   
    ,      ,      
   0   ).
@memo   
@param node       
@param fragmentfn    .    NULL,    
	     .
@return SUCCESS       
@see db_FragmentByDefault ErrorCodes dictionary_t DatabaseConfig
*/
extern int db_InitDatabase(int node, fragment_fn_t fragmentfn)
{
	int i;
#ifdef RANDOM_DATABASE
	j, k;
	FILE * f;
	tuple_t tuple;
#endif

	START("db_InitDatabase");
	if (node<0 || node>=FRAGMENTS_IN_RELATION)
		return EINVAL;
	initflag=TRUE;

	/*  -EOF */
	EOFtuple=(tuple_t) calloc(1, sizeof(domain_t));
	assert(EOFtuple!=NULL);
	for (i=0; i<ATTRIBUTES_IN_RELATION; i++)
		EOFtuple[i]=EOFmarker;

	/*   */
	DICTIONARY.whoami=node;
	for (i=0; i<RELATIONS_IN_DATABASE; i++) {
		DICTIONARY.relations[i].relation=i;
		DICTIONARY.relations[i].fragment=DICTIONARY.whoami;
		strcpy(DICTIONARY.relations[i].filename, RELATIONS_FILENAMES[i][DICTIONARY.whoami]);
		DICTIONARY.relations[i].tuples=TUPLES_IN_FRAGMENT[i];
		DICTIONARY.relations[i].ary=ATTRIBUTES_IN_RELATION;
		DICTIONARY.relations[i].fragmentattr=FRAGMENTATION_ATTRIBUTES[i];
		if (fragmentfn==NULL)
			DICTIONARY.relations[i].fragmentfn=db_FragmentByDefault;
		else
			DICTIONARY.relations[i].fragmentfn=fragmentfn;
	}

#ifdef RANDOM_DATABASE
	/*   */
	for (i=0; i<RELATIONS_IN_DATABASE; i++) {
		f=fopen(DICTIONARY.relations[i].filename, "wt");
		assert(f!=NULL);
		/*   */
		for (j=0; j<DICTIONARY.relations[i].tuples; j++) {
			tuple=(tuple_t) calloc (ATTRIBUTES_IN_RELATION, sizeof(domain_t));
			assert(tuple!=NULL);
			/*     */
			tuple[0]=j+DICTIONARY.whoami*DICTIONARY.relations[i].tuples;
			/*     */
			for (k=1; k<ATTRIBUTES_IN_RELATION; k++) 
				tuple[k]=rand()%DOMAIN_VALUE_MAX;
			/*     */
			while (DICTIONARY.relations[i].fragmentfn(tuple, DICTIONARY.relations[i].fragmentattr)!=DICTIONARY.whoami)
				tuple[DICTIONARY.relations[i].fragmentattr]=rand()%DOMAIN_VALUE_MAX;
			for (k=0; k<ATTRIBUTES_IN_RELATION; k++)
				fprintf(f, "%d\t", tuple[k]);
			fprintf(f, "\n");

			free(tuple);
		}
		fclose(f);
	}
#endif
	STOP("db_InitDatabase");
	return SUCCESS;
}

/*
@memo      
@return         NULL   
@see dictionary_t 
*/

dictionary_t * db_DatabaseDictionary(void)
{
	if (!initflag)
		return NULL;
	return &DICTIONARY;
}

/*
        .
@memo   
@param tuple   
@param fragmentattr      (  0)
@return   (..    ,   
  ,     ) 
  0..FRAGMENTS_IN_RELATION-1
@see db_InitDatabase fragment_fn_t domain_t FRAGMENTATION_ATTRIBUTES
*/
int db_FragmentByDefault(tuple_t tuple, int fragmentattr)
{
	assert(tuple!=NULL);
	assert(!IsEOF(tuple));
	assert(fragmentattr>=0);
	return tuple[fragmentattr]%FRAGMENTS_IN_RELATION;
}

/*@}*/

/*@}*/
