/*
    .
          .
        .
      :    
   /     ,   
/   .     
( ).

.
       MPI.
@memo  
@name MessageManager_Implementation
@author .. , .. 
@version 1.0
*/

#include "msgmgr.h"		/*  */
#include "errcode.h"	/*   */
#include "common.h"		/*     */
#include "mpi.h"		/*  MPI */
#include "debug.h"		/*    */
#include "mpiassrt.h"
#include <stdlib.h>
#include <stdio.h>


//#define NDEBUG
#undef NDEBUG

/*@{*/

/*
     
@name MessageManager_Implementation_Constants
@memo   
@see MessageManager_Implementation_Functions
*/

/*@{*/

/*
@memo   
*/
#define OPTAB_LEN	(255)

/*@}*/


/*
     
@name MessageManager_Implementation_Types
@memo   
*/

/*@{*/

/*
@memo 
*/
typedef struct {
	mid_t operation;
	MPI_Request request;
} optabelem_t;

/*
@memo  
*/
typedef  optabelem_t optab_t[OPTAB_LEN];

/*@}*/


/*
     
@name MessageManager_Implementation_Types
@memo   
*/

/*@{*/

/*
@memo  
*/
static int initflag=FALSE;

/*
@memo  
*/
static optab_t OPTAB;

/*
@memo   
*/
static int whoami;

/*@}*/

/*
     
@name MessageManager_Functions
@memo   
@see MessageManager_Constants
*/

/*@{*/

/*
    .
           .
          mm_WhoAmI.
      .
@memo   
@return SUCCESS       
@see mm_DoneMessageManager mm_WhoAmI mm_SendMessage mm_RecvMessage
*/
int mm_InitMessageManager()
{
	int i, res;

	START("mm_InitMessageManager");
	if (initflag)
		return SUCCESS;
	initflag=TRUE;
	for (i=0; i<OPTAB_LEN; i++)
		OPTAB[i].operation=NIL;
	res=MPI_Init(NULL, NULL);
	MPI_Assert(res==MPI_SUCCESS);
	res=MPI_Comm_rank(MPI_COMM_WORLD, &whoami);
	MPI_Assert(res==MPI_SUCCESS);
	res=MPI_Barrier(MPI_COMM_WORLD);/*    */
	MPI_Assert(res==MPI_SUCCESS);
	RETURN_SUCCESS("mm_InitMessageManager");
	STOP("mm_InitMessageManager");
	return SUCCESS;
}

/*
@memo    
@return       >=0    
@see mm_InitMessageManager
*/
int mm_WhoAmI(void)
{
	START("mm_WhoAmI");
	if (!initflag)
		return ENOINIT;
	RETURN_INT("mm_WhoAmI", whoami);
	STOP("mm_WhoAmI");
	return whoami;
}

/*
          .
     (..  ).
      :    
   /     ,   
/   .     
( ).
@memo   
@param destnode  -
@param buffer    
@param len    
@return   >=0       
@see ErrorCodes mm_RunRecv mm_IsDone
*/
mid_t mm_RunSend(int port, int destnode, void * buffer, int len)
{
	int i, res;

	START("mm_RunSend");
	if (!initflag)
		return ENOINIT;
	if (buffer==NULL || len<=0)
		return EINVAL;
	if (destnode==whoami || destnode<0 || destnode>=NODES_IN_CLUSTER)
		return EINVAL;

	for (i=0; i<OPTAB_LEN; i++)
		if (OPTAB[i].operation==NIL) {
			OPTAB[i].operation=i;
			/* 
			int MPI_Isend
			int MPI_Issend
			(void *buf, int count, MPI_Datatype datatype, int dest, int msgtag, MPI_Comm comm, MPI_Request *request) 
			*/
			res=MPI_Issend(buffer, len, MPI_BYTE, destnode, port, MPI_COMM_WORLD, &OPTAB[i].request);
			MPI_Assert(res==MPI_SUCCESS);
			RETURN_INT("mm_RunSend", OPTAB[i].operation);
			STOP("mm_RunSend");
			return OPTAB[i].operation;
		}
	RETURN_EOVRFLOW("mm_RunSend");
	STOP("mm_RunSend");
	return EOVRFLOW;
}

/*
           .
      (..   ).
      :    
   /     ,   
/   .     
( ).
@memo   
@param srcnode  -
@param buffer    
@param len    
@return   >=0       
@see ErrorCodes mm_RunRecv mm_IsDone
*/
mid_t mm_RunRecv(int port, int srcnode, void * buffer, int len)
{
	int i, res;

	START("mm_RunRecv");
	if (!initflag)
		return ENOINIT;
	if (buffer==NULL || len<=0)
		return EINVAL;
	if (srcnode==whoami || srcnode<0 || srcnode>=NODES_IN_CLUSTER)
		return ESRCH;

	for (i=0; i<OPTAB_LEN; i++)
		if (OPTAB[i].operation==NIL) {
			OPTAB[i].operation=i;
			/*
			int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, 
						int source, int msgtag, MPI_comm comm, MPI_Request *request) 
			*/
			res=MPI_Irecv(buffer, len, MPI_BYTE, srcnode, port, MPI_COMM_WORLD, &OPTAB[i].request);
			MPI_Assert(res==MPI_SUCCESS);
			RETURN_INT("mm_RunRecv", OPTAB[i].operation);
			STOP("mm_RunRecv");
			return OPTAB[i].operation;
		}
	RETURN_EOVRFLOW("mm_RunRecv");
	STOP("mm_RunRecv");	
	return EOVRFLOW;
}

/*
         
@memo   
@param operation  
@return 1,   , 0,      
       
@see mm_RunSend mm_RunRecv
*/
int mm_IsDone(mid_t operation)
{
	int i, res, flag;
	MPI_Status status;

	START("mm_IsDone");
	if (!initflag) {
		RETURN_ENOINIT("mm_IsDone");
		return ENOINIT;
	}
	if (operation<0) {
		RETURN_EINVAL("mm_IsDone");
		return EINVAL;
	}
	for (i=0; i<OPTAB_LEN; i++)
		if (OPTAB[i].operation==operation) {
			res=MPI_Test(&OPTAB[operation].request, &flag, &status);
			MPI_Assert(res==MPI_SUCCESS);
			if (flag)
				OPTAB[i].operation=NIL;
			RETURN_INT("mm_IsDone", flag);
			STOP("mm_IsDone");
			return flag;
		}
	RETURN_ESRCH("mm_IsDone");
	STOP("mm_IsDone");
	return ESRCH;
}


/*
     , ..       
  ,        .
@memo  
@return 0       
@see mm_InitMessageManager mm_DoneMessageManager
*/
int mm_Synchronize(void)
{
	int res;

	START("mm_Synchronize");
	if (!initflag)
		return ENOINIT;
	res=MPI_Barrier(MPI_COMM_WORLD);
	MPI_Assert(res==MPI_SUCCESS);
	RETURN_SUCCESS("mm_Synchronize");
	STOP("mm_Synchronize");
	return SUCCESS;
}

/*
        ; 
    (     )  .

   mm_DoneMessageManager     {\Ref ENOINIT}, ..
    {\Ref mm_InitMessageManager}.
@memo   
@return 0       
@see mm_InitMessageManager ErrorCodes
*/
int mm_DoneMessageManager()
{
	int res;

	START("mm_DoneMessageManager");
	if (!initflag)
		return ENOINIT;
	initflag=FALSE; /*   mm_DoneMessageManager   mm_InitMessageManager */
	res=MPI_Barrier(MPI_COMM_WORLD);/*    */
	MPI_Assert(res==MPI_SUCCESS);
	res=MPI_Finalize();
	MPI_Assert(res==MPI_SUCCESS);
	RETURN_SUCCESS("mm_DoneMessageManager");
	STOP("mm_DoneMessageManager");
	return SUCCESS;
}

/*@}*/

/*@}*/

